#include "LotusHashExtractor.h"


/* Globals */
static HANDLE hFile,hFileMap;


/* Open file and map it to memory */
LPVOID MapFile(LPSTR szFileName,PSIZE_T fileSize)
{
	LARGE_INTEGER size;
	LPVOID lpFileData;

	hFile = CreateFile(szFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hFile == INVALID_HANDLE_VALUE)
		return NULL;

	if(!GetFileSizeEx(hFile,&size) || (size.QuadPart > UINT_MAX))
	{
		CloseHandle(hFile);
		return NULL;
	}
	
	*fileSize = size.LowPart;

	hFileMap = CreateFileMapping(hFile,NULL,PAGE_READONLY,0,size.LowPart,NULL); 
	if(!hFileMap)
	{
		CloseHandle(hFile);
		return NULL;
	}

	lpFileData = MapViewOfFile(hFileMap,FILE_MAP_READ,0,0,NULL);

	if(!lpFileData)
	{
		CloseHandle(hFileMap);
		CloseHandle(hFile);
	}
	return lpFileData;
}


/* Unmap file from memory and cleanup */
void UnMapFile(LPCVOID lpFileBytes)
{
	UnmapViewOfFile(lpFileBytes);
	CloseHandle(hFileMap);
	CloseHandle(hFile);
}


/*
 * Extract lotus domino user hash from data
 * returns
 *  1 if success
 *  0 if file format is invalid
 */
int ReadDominoHashLine(LPBYTE pUserId,SIZE_T dataSize,LPSTR szHashLine)
{
	SIZE_T i;
	SIZE_T wUserBlobSize;

	if(dataSize < 0xD8)
		return 0;

	wUserBlobSize = *(LPWORD)(pUserId + 0xD6);	/* Should be user id ciphered blob size */

	if((wUserBlobSize < 0x10) || (wUserBlobSize > 0x64))	/* Blob size is variable but there are some limits */
		return 0;

	if((wUserBlobSize + 0xD8) > dataSize)
		return 0;

	for(i=0;i<wUserBlobSize;++i)
		wsprintf(szHashLine+(i<<1),"%02X",pUserId[0xD8+i]);

	return 1;
}


/*
 * Hash line extraction
 */
int ExtractLotusDominoHashLine(LPSTR szFileName,LPSTR szHashLine)
{
	LPBYTE pFileData;
	SIZE_T fileSize = 0;
	int ret;

	/* Try to open file map */
	pFileData = (LPBYTE)MapFile(szFileName,&fileSize);

	if(!pFileData)
		return -1;

	if(!fileSize)
	{
		UnMapFile(pFileData);
		return 0;
	}
	
	/* Extract what we need */
	ret = ReadDominoHashLine(pFileData,fileSize,szHashLine);

	/* Close file mapping */
	UnMapFile(pFileData);

	return ret;
}
